/**
 *  @company Reborn LTD
 *  @device  PIC12F1572
 *  @file    main.c
 *  @author  Berran Remzi (berranremzi@gmail.com)
 *  @date	 15.01.2019
 *  @version 1.0
 *
 *  @todo
 *  @see
 *  @summary
 *       PLC based on PIC16F1619 with 16MHz External Oscillator
 * 
 *  @description
 * 
 * Slow Digital Inputs: 8 (IN1, ..., IN8)
 * Fast Digital Inputs: 2 (IN9, IN10)
 * Relay Outputs: 5 (OUT1, ..., OUT5)
 * PNP Isolated Outputs: 3 (OUT6, OUT7, OUT8)
 * Fast PWM Isolated Outputs: 2 (OUT9, OUT10)
 * 
 * Analog Inputs: 2 (A1, A2) 
 *                              
 */

/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.65.2
        Device            :  PIC16F1619
        Driver Version    :  2.00
 */

#include "mcc_generated_files/mcc.h"
#include "plc_sirius/PLC_SIRIUS.h"
#include "plc_sirius/millis.h"

void test1();
void test2();
void test3();
uint8_t fade(int16_t*, bool*);

/*
                         Main application
 */
void main(void) {
    // initialize the device
    SYSTEM_Initialize();

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();
    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    TMR0_SetInterruptHandler(call_millis);
    __delay_ms(250);

    test1();
    test2();
    test3();

    volatile uint8_t rxData;

    while (1) {

        if (digitalRead(IN9)) {
            static int16_t pwm1;
            static bool dir1;
            analogWrite(OUT9, fade(&pwm1, &dir1));
            TX_Disable();
        } else {
            analogWrite(OUT9, analogRead(A1) >> 2);

            TX_Enable();

            for (uint8_t j = 0; j < 8; j++) {
                if (digitalRead(16 + j)) {
                    digitalWrite(j, HIGH);
                } else {
                    digitalWrite(j, LOW);
                }
            }
        }

        if (digitalRead(IN9)) {
            if (EUSART_is_rx_ready()) {
                rxData = EUSART_Read();
                if (EUSART_is_tx_ready()) {
                    if (rxData) {
                        for (int i = 0; i < 8; i++) {
                            digitalWrite(i, (rxData >> i) & 1U);
                        }
                    }
                }
            }
        }


        if (digitalRead(IN10)) {
            static int16_t pwm2;
            static bool dir2;
            analogWrite(OUT10, fade(&pwm2, &dir2));

        } else {
            analogWrite(OUT10, analogRead(A2) >> 2);
        }



        if (SoftTimerInterrupt10ms()) {
            ;
        }
        if (SoftTimerInterrupt50ms()) {
            ;
        }
        if (SoftTimerInterrupt100ms()) {
            ;
        }

        if (SoftTimerInterrupt500ms()) {
            ;
        }

        if (SoftTimerInterrupt1s()) {
            printf("Hello World!!!\n");
        }

        if (SoftTimerInterrupt5s()) {
            ;
        }
        __delay_ms(5);

    }
}

void test1() {
    for (uint8_t t = 0; t < 8; t++) {

        digitalWrite(t, HIGH);
        __delay_ms(500);
        digitalWrite(t, LOW);
    }
}

void test2() {
    for (uint8_t t = 0; t < 8; t++) {

        digitalWrite(t, HIGH);
    }
    __delay_ms(500);
}

void test3() {
    for (uint8_t t = 0; t < 8; t++) {

        digitalWrite(t, LOW);
    }
    __delay_ms(500);
}

uint8_t fade(int16_t* _pwmValue, bool* _dir) {
    if (*_dir) {
        (*_pwmValue)--;
    } else {
        (*_pwmValue)++;
    }

    if (*_pwmValue >= 255) {
        *_dir = 1;
    }
    if (*_pwmValue <= 0) {
        *_dir = 0;
    }

    return (uint8_t) * _pwmValue;
}
/**
 End of File
 */