/*
 * File:   PLC_SIRIUS.c
 * Author: Berran
 *
 * Created on January 14, 2019, 9:53 PM
 */

#include "../mcc_generated_files/mcc.h"
#include <xc.h>
#include "HC595.h"
#include "PLC_SIRIUS.h"

void analogWrite(uint8_t _pin, uint16_t _pwmValue) {
    switch (_pin) {
        case 9:
            PWM4_LoadDutyValue(_pwmValue);
            break;
        case 8:
            PWM3_LoadDutyValue(_pwmValue);
            break;
        default: break;
    }
}

uint16_t analogRead(uint8_t _pin) {
    switch (_pin) {
        case 32: return ADC_GetConversion(ADC1);
            break;
        case 33: return ADC_GetConversion(ADC2);
            break;
        default: break;
    }
    return 0;
}

void digitalWrite(uint8_t _pin, bool _state) {
    if (_pin > 15) return;

    if (_state) {
        setOutImageBit(_pin);
    } else {
        clearOutImageBit(_pin);
    }

    if (_pin == 8) {
        PORTAbits.RA0 = _state;
    }
    if (_pin == 9) {
        PORTAbits.RA1 = _state;
    }

    if (_pin < 8) {
        updateHC595();
    }
}

bool digitalRead(uint8_t _pin) {

    if (_pin > 15 && _pin < 24) {
        inImage = (uint8_t) HC165ReadValue(HC595_LSB_FIRST);
    } else if (_pin > 23 && _pin < 32) {
        switch (_pin) {
            case 24:
                if (PORTBbits.RB7) {

                    inImage |= 1UL << convertInputNumberToBit(_pin);
                    return 1;
                }
                break;
            case 25:
                if (PORTAbits.RA2) {
                    inImage |= 1UL << convertInputNumberToBit(_pin);
                    return 1;
                }
                break;
            default: break;
        }
    } else {
        return 0;
    }

    return inImage & convertInputNumberToBit(_pin);
}

uint8_t HC165ReadValue(uint8_t direction) {
    uint8_t value = 0x00;
    HC165TriggerOutput();
    for (uint8_t i = 0; i < 8; i++) {

        if (HC165_Q7) {
            if (direction == HC595_MSB_FIRST) {
                value |= 1U << i;
            } else {
                value |= 0x80 >> i;
            }
        }
        HC165ToggleClk();
    }
    return value;
}

void HC165TriggerOutput() {

    HC165_PL = 0;
    __delay_us(DELAY_READ);
    HC165_PL = 1;
}

void HC165ToggleClk() {

    HC165_CP = 1;
    __delay_us(DELAY_READ);
    HC165_CP = 0;
    __delay_us(DELAY_READ);
}

static uint8_t convertInputNumberToBit(uint8_t inNumber) {
    uint8_t bitNumber = 0x00;
    switch (inNumber) {
        case 16: bitNumber = 3;
            break;
        case 17: bitNumber = 2;
            break;
        case 18: bitNumber = 1;
            break;
        case 19: bitNumber = 0;
            break;
        case 20: bitNumber = 4;
            break;
        case 21: bitNumber = 5;
            break;
        case 22: bitNumber = 6;
            break;
        case 23: bitNumber = 7;
            break;
        case 24: bitNumber = 8;
            break;
        case 25: bitNumber = 9;
            break;
        default: break;
    }
    return 1 << bitNumber;
}

static uint8_t convertOutNumberToBit(uint8_t outNumber) {
    uint8_t bitNumber = 0x00;
    switch (outNumber) {
        case 0: bitNumber = 0;
            break;
        case 1: bitNumber = 3;
            break;
        case 2: bitNumber = 2;
            break;
        case 3: bitNumber = 1;
            break;
        case 4: bitNumber = 7;
            break;
        case 5: bitNumber = 6;
            break;
        case 6: bitNumber = 5;
            break;
        case 7: bitNumber = 4;
            break;
        case 8: bitNumber = 8;
            break;
        case 9: bitNumber = 9;

            break;
        default: break;
    }

    return 1 << bitNumber;
}

void updateHC595() {

    HC595SendValue((uint8_t) outImage, HC595_LSB_FIRST);
    HC595TriggerOutput();
}

static void setOutImageBit(uint8_t outNumber) {

    outImage |= convertOutNumberToBit(outNumber);
}

static void clearOutImageBit(uint8_t outNumber) {

    outImage &= ~convertOutNumberToBit(outNumber);
}

int16_t map(int16_t x, int16_t in_min, int16_t in_max, int16_t out_min, int16_t out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}